#include <QtGui>
#include <QDialog>
#include <taglib/tag.h>
#include <taglib/fileref.h>
#include <taglib/id3v1tag.h>
#include <taglib/id3v2tag.h>
#include <taglib/tfile.h>
#include <taglib/mpegfile.h>

#include "detailsdialog.h"
#include "settingsdialog.h"
#include "decoder_mad.h"
#include "id3tag.h"
#include "decodermadfactory.h"

// DecoderMADFactory

bool DecoderMADFactory::supports(const QString &source) const
{
    QString ext = source.right(4).toLower();
    return ext == ".mp1" || ext == ".mp2" || ext == ".mp3";
}

const QString &DecoderMADFactory::name() const
{
    static QString name (tr("MPEG Plugin"));
    return name;
}

const QString &DecoderMADFactory::filter() const
{
    static QString filter("*.mp1 *.mp2 *.mp3"); 
    return filter;
}

const QString &DecoderMADFactory::description() const
{
    static QString desc(tr("MPEG Files"));
    return desc;
}

Decoder *DecoderMADFactory::create(QObject *parent, QIODevice *input, Output *output, bool deletable)
{

    //if (deletable)
    {
        qDebug ("create FACTORY" );
        return new DecoderMAD(parent, this, input, output);
    }
    static DecoderMAD *decoder = 0;

    if (! decoder)
    {
        decoder = new DecoderMAD(parent, this, input, output);
    }
    else
    {
        decoder->setInput(input);
        decoder->setOutput(output);
    }

    return decoder;
}

FileTag *DecoderMADFactory::createTag(const QString &source)
{
    FileTag *tag = new ID3Tag(source);
    return tag;
}

void DecoderMADFactory::showDetails(QWidget *parent, const QString &path)
{
    DetailsDialog *d = new DetailsDialog(parent, path);
    d -> show();
}

void DecoderMADFactory::showSettings(QWidget *parent)
{
    SettingsDialog *s = new SettingsDialog(parent);
    s -> show();
}

Q_EXPORT_PLUGIN(DecoderMADFactory)
