//Added by qt3to4:
//#include <QCustomEvent>
// Copyright (c) 2000-2001 Brad Hughes <bhughes@trolltech.com>
//
// Use, modification and distribution is allowed without limitation,
// warranty, or liability of any kind.
//

#ifndef   __output_h
#define   __output_h

class Output;
//class OutputEvent;

#include <QObject>
#include <QThread>
#include <QEvent>
#include <QList>
#include <QIODevice>
#include <outputfactory.h>

#include "recycler.h"

class Visual;

/*
class OutputEvent : public QCustomEvent
{
public:
    enum Type { Playing = (User + 200), Buffering, Info, Paused,
		Stopped, Error };
 
    OutputEvent(Type t)
	: QCustomEvent(t), error_msg(0), elasped_seconds(0), written_bytes(0),
	  brate(0), freq(0), prec(0), chan(0)
    { ; }
 
    OutputEvent(long s, unsigned long w, int b, int f, int p, int c)
	: QCustomEvent(Info), error_msg(0), elasped_seconds(s), written_bytes(w),
	  brate(b), freq(f), prec(p), chan(c)
    { ; }
 
    OutputEvent(const QString &e)
	: QCustomEvent(Error), elasped_seconds(0), written_bytes(0),
	  brate(0), freq(0), prec(0), chan(0)
    {
	error_msg = new QString(e.utf8());
    }
 
 
    ~OutputEvent()
    {
	if (error_msg)
	    delete error_msg;
    }
 
    const QString *errorMessage() const { return error_msg; }
 
    const long &elapsedSeconds() const { return elasped_seconds; }
    const unsigned long &writtenBytes() const { return written_bytes; }
    const int &bitrate() const { return brate; }
    const int &frequency() const { return freq; }
    const int &precision() const { return prec; }
    const int &channels() const { return chan; }
 
 
private:
    QString *error_msg;
 
    long elasped_seconds;
    unsigned long written_bytes;
    int brate, freq, prec, chan;
};
*/


class OutputState
{
public:

    enum Type { Playing, Buffering, Info, Paused, Stopped, Error };
    
    OutputState()
            :m_type(Stopped),  m_error_msg(0), m_elasped_seconds(0), 
             m_written_bytes(0),m_brate(0), m_freq(0), m_prec(0), m_chan(0)
    {}

    OutputState(Type t)
            :m_type(t),  m_error_msg(0), m_elasped_seconds(0),
             m_written_bytes(0), m_brate(0), m_freq(0), m_prec(0), m_chan(0)
    {}
    OutputState(long s, unsigned long w, int b, int f, int p, int c)
            : m_type(Info), m_error_msg(0), m_elasped_seconds(s),
              m_written_bytes(w), m_brate(b), m_freq(f), m_prec(p), m_chan(c)
    {}
    OutputState(const QString &e)
	: m_type(Error), m_elasped_seconds(0), m_written_bytes(0),
	  m_brate(0), m_freq(0), m_prec(0), m_chan(0)
    {
	m_error_msg = new QString(e.toUtf8());
    }
    ~OutputState()
    {
       if (m_error_msg)
           delete m_error_msg;
    }

    const QString *errorMessage() const { return m_error_msg; }
 
    const long &elapsedSeconds() const { return m_elasped_seconds; }
    const unsigned long &writtenBytes() const { return m_written_bytes; }
    const int &bitrate() const { return m_brate; }
    const int &frequency() const { return m_freq; }
    const int &precision() const { return m_prec; }
    const int &channels() const { return m_chan; }
    const Type &type() const { return m_type; }

private:
    Type m_type;
    QString *m_error_msg;
    long m_elasped_seconds;
    unsigned long m_written_bytes;
    int m_brate, m_freq, m_prec, m_chan;
};




class Output : public QThread
{
    Q_OBJECT
public:
    Output( QObject * parent = 0);
    ~Output();

    Recycler *recycler()
    {
        return &r;
    }

    void addListener(QObject *);
    void removeListener(QObject *);

    void addVisual(Visual *);
    void removeVisual(Visual *);

    QMutex *mutex()
    {
        return &mtx;
    }

    void setBufferSize(unsigned int sz)
    {
        bufsize = sz;
    }
    unsigned int bufferSize() const
    {
        return bufsize;
    }
    //****
    long elapsed_seconds;
    unsigned long written_bytes;
    int brate, freq, prec, chan;
    //****
    // abstract
    virtual bool initialized() const = 0;
    virtual bool initialize() = 0;
    virtual void configure(long, int, int, int) = 0;
    virtual void pause() = 0;
    virtual void stop() = 0;
    virtual long written() = 0;
    virtual long latency() = 0;
    virtual void seek(long) = 0;
    
    //propertyes
    const long &elapsedSeconds() const
    {
        return elapsed_seconds;
    }
    //static
    
    static void registerFactory(OutputFactory *);
    static Output *create(QObject *);
    static QList<OutputFactory*> *outputFactories();
    static QStringList outputFiles();

signals:
    void stateChanged(OutputState *);



protected:
    void dispatch(const OutputState&);
    void error(const QString &e);
    void dispatchVisual(Buffer *, unsigned long, int, int);
    void prepareVisuals();


private:
    OutputState m_state;
    QMutex mtx;
    Recycler r;
    QList<QObject *> listeners;
    QList<Visual *> visuals;

    unsigned int bufsize;


};


#endif // __output_h
