/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                      *
 *   forkotov02@hotmail.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QtGui>
#include <QFileDialog>
#include <QDir>
#include <QAction>

#include "textscroller.h"
#include "mainwindow.h"
#include "constants.h"
#include "fileloader.h"
#include "skin.h"
#include "playlist.h"
#include "playlistmodel.h"
#include "configdialog.h"
#include "dock.h"
#include "eqwidget.h"
#include "mainvisual.h"



MainWindow::MainWindow(QWidget *parent)
        : QMainWindow(parent)
{
    input = 0;
    output = 0;
    decoder = 0;
    m_vis = 0;
    seeking = FALSE;

    Skin* skin = new Skin(this);
    Dock *dock = new Dock(this);
    dock->setMainWidget(this);

    setWindowFlags(Qt::FramelessWindowHint);
    //setWindowFlags(Qt::Tool);
    setFixedSize (275,116);

    display = new MainDisplay(this);
    setCentralWidget(display);
    display->show();
    display->setFocus ();

    m_playlist = new PlayList(this);
    m_playListModel = new PlayListModel(this);
    dock->addWidget(m_playlist);

    m_equalizer = new EqWidget(this);
    dock->addWidget(m_equalizer);
    connect(m_equalizer, SIGNAL(valueChanged()), SLOT(updateEQ()));

    m_playlist->setModel(m_playListModel);
    QAction *act = new QAction("&Settings",this);
    act->setShortcut(tr("Ctrl+P"));
    connect(act, SIGNAL(triggered()), SLOT(showSettings()));
    addAction(act);
    m_playlist->addAction(act);
    m_equalizer->addAction(act);

    m_titlebar = new TitleBar(this);
    m_titlebar->move(0,0);
    m_titlebar->show();
    m_titlebar->setActive(TRUE);

    readSettings();
    dock->updateDock();

    display->setEQ(m_equalizer);
    display->setPL(m_playlist);

    m_vis = MainVisual::getPointer();

}


MainWindow::~MainWindow()
{
    stopAll();


}

void MainWindow::play()
{
    int outputBuffersize = 128;
    stopAll();
    if(m_playListModel->count() == 0)
        return;
    QString s = m_playListModel->currentItem()->path();
    if (s.isEmpty())
        return;
    input = new QFile(s);

    if (output)
        fprintf(stderr, "output not deleted!\n");

    //output = new AudioOutput(this, "/dev/dsp");
    output = Output::create(this);
    output -> setBufferSize(outputBuffersize*1024); //TODO: move to output plugin
    //output -> setStackSize(outputBuffersize*1024);
    output -> addListener(this);
    if (! output->initialize())
        return;

    connect(output, SIGNAL(stateChanged(OutputState *)),
                             SLOT(showOutputState(OutputState *)));

    /* visualiation*/
    //m_vis = new MainVisual(0,"dsd");
    //m_vis->show();
    m_vis->setOutput(output);
    output->addVisual(m_vis);

    if (decoder && ! decoder->factory()->supports(s))
        decoder = 0;


    if (! decoder)
    {
        qDebug ("creating decoder");
        decoder = Decoder::create(this, s, input, output);

        if (! decoder)
        {
            qWarning("MQ3: unsupported fileformat\n");
            stopAll();
            return;
        }
        qDebug ("ok");
        decoder->setBlockSize(globalBlockSize);
        decoder->addListener(this);
        connect(decoder, SIGNAL(stateChanged(DecoderState *)),
                SLOT(showDecoderState(DecoderState *)));
        updateEQ();
    }
    else
    {
        decoder->setInput(input);
        decoder->setOutput(output);
    }


    if (decoder->initialize())
    {

        display->setTime(0);
        display->setMaxTime(int(decoder->lengthInSeconds()));
        output->start();
        decoder->start();
    }
}

void MainWindow::seek(int pos)
{
    if (output && output->isRunning())
    {
        output->mutex()->lock();
        output->seek(pos);

        if (decoder && decoder->isRunning())
        {
            decoder->mutex()->lock();
            decoder->seek(pos);

            /* if (mainvisual)
             {
                 mainvisual->mutex()->lock()
                 ;
                 mainvisual->prepare();
                 mainvisual->mutex()->unlock();
             }*/

            decoder->mutex()->unlock();
        }

        output->mutex()->unlock();
    }
}

void MainWindow::pause(void)
{
    if (output)
    {
        //output->mutex()->lock();
        output->pause();
        //output->mutex()->unlock();
    }

    // wake up threads
    if (decoder)
    {
        decoder->mutex()->lock();
        decoder->cond()->wakeAll();
        decoder->mutex()->unlock();
    }

    if (output)
    {
        output->recycler()->mutex()->lock();
        output->recycler()->cond()->wakeAll();
        output->recycler()->mutex()->unlock();
    }
}

void MainWindow::stop()
{
    if (decoder && decoder->isRunning())
    {
        decoder->mutex()->lock();
        decoder->stop();
        decoder->mutex()->unlock();
    }

    if (output && output->isRunning())
    {
        output->mutex()->lock();
        output->stop();
        output->mutex()->unlock();
    }

    // wake up threads
    if (decoder)
    {
        decoder->mutex()->lock();
        decoder->cond()->wakeAll();
        decoder->mutex()->unlock();
    }

    if (output)
    {
        output->recycler()->mutex()->lock();
        output->recycler()->cond()->wakeAll();
        output->recycler()->mutex()->unlock();
    }

    if (decoder)
        decoder->wait();

    if (output)
        output->wait();

    // mainvisual->setDecoder(0);
    //mainvisual->setOutput(0);

    if(output)
    {
       delete output;
       output = 0;
    }
    if(input)
    { 
       delete input;
       input = 0;
    }
    display->setTime(0);
    if(decoder)
    {
       delete decoder;
       decoder = 0;
    }
    /*if(m_vis)
    {
       delete m_vis;
       m_vis = 0;
    }*/
}
void MainWindow::next()
{
    if (!m_playListModel->next())
    {
        stop();
        return;
    }
    m_playlist->update();
    if (output)
    {
        stop();
        play();
    }
}
void MainWindow::previous()
{
    if(!m_playListModel->previous())
        return;

    m_playlist->update();
    if (decoder && decoder->isRunning())
    {
        stop();
        play();
    }
}


void MainWindow::stopAll()
{
    stop();
    if (decoder)
    {
        //delete decoder;
        //decoder->removeListener(this);
        //decoder = 0;

    }

}

void MainWindow::updateEQ()
{
   if (decoder)
   {
       int b[10];
       for (int i=0; i<10; ++i)
            b[i] = m_equalizer->gain(i);
       decoder->mutex()->lock();
       decoder->setEQ(b, m_equalizer->preamp());
       decoder->setEQEnabled(m_equalizer->isEQEnabled());
       decoder->mutex()->unlock();
   }
}

void MainWindow::showOutputState(OutputState *st)

{
    if(!output || seeking)
        return;
  
    display->setInfo(st);
    /*switch ((int) st->type())
    {
    case OutputState::Info:
        {
            if (seeking)
                break;
            display->setTime(st->elapsedSeconds());
            display->setInfo(st);
            display->update();
            break;
        }
    case OutputState::Playing:
        {
            //ui.label->setText("Playing");
            break;
        }
    case OutputState::Buffering:
        {
            //ui.label->setText("Buffering");
            break;
        }
    case OutputState::Paused:
        {
            //ui.label->setText("Paused");
            break;
        }
    case OutputState::Stopped:
        {
            //ui.label->setText("Stopped");
            break;
        }

    }*/

}
void MainWindow::showDecoderState(DecoderState *st)
{
    if (!decoder)
        return;

    switch ((int) st->type())
    {
    case DecoderState::Finished:
        {
            //c->sendNext();
            next();
            break;
        }
    }
}

void MainWindow::closeEvent ( QCloseEvent *)
{
    writeSettings();
}

void MainWindow::addDir()
{
    QString s = QFileDialog::getExistingDirectory(
                    this,
                    "Choose a directory",
                    m_lastDir,
                    QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly);

    if(s.isEmpty())
       return;
    m_lastDir = s+"../";
    FileLoader fl(this);
    QFileInfoList list = fl.getFileList(s);

    for (int i = 0; i < list.size(); ++i)
    {
        QFileInfo fileInfo = list.at(i);
        if(Decoder::supports(fileInfo.absoluteFilePath ()))
            m_playListModel->load(new MediaFile(fileInfo.absoluteFilePath ()));

    }
    //ui.listWidget_songs->sortItems();
    return;
}

void MainWindow::addFile()
{
    QStringList files = QFileDialog::getOpenFileNames(
                            this,
                            "Select one or more files to open",
                             m_lastDir,
                             Decoder::filter());
    if(files.isEmpty ())
      return;
    foreach(QString s, files)
    m_playListModel->load(new MediaFile(s));
    m_lastDir = files.at(0);
    return;
}

void MainWindow::clear()
{
    m_playListModel->clear();
}

void MainWindow::startSeek()
{
    seeking = TRUE;
}

void MainWindow::endSeek()
{
    seeking = FALSE;
}

/*void MainWindow::mousePressEvent(QMouseEvent * e)
{
    m_titlebar->setActive(TRUE);
}

void MainWindow::enterEvent ( QEvent * event )
{
    m_titlebar->setActive(TRUE);
}

void MainWindow::leaveEvent ( QEvent * event )
{
    m_titlebar->setActive(FALSE);
}*/

void MainWindow::changeEvent ( QEvent * event )
{
  if (event->type() == QEvent::ActivationChange)
  {
     m_titlebar->setActive(isActiveWindow());
  } 
}

void MainWindow::readSettings()
{
    QSettings settings(QDir::homePath()+"/.qmmp/qmmprc", QSettings::IniFormat);

    settings.beginGroup("MainWindow");
    //geometry
    move(settings.value("pos", QPoint(100, 100)).toPoint());
    //last directory
    m_lastDir = settings.value("last_dir","/").toString();
    settings.endGroup();
    show();
    //visibility
    m_playlist->setVisible(settings.value("Playlist/visible",TRUE).toBool());
    m_equalizer->setVisible(settings.value("Equalizer/visible",TRUE).toBool());
}

void MainWindow::writeSettings()
{
    QSettings settings(QDir::homePath()+"/.qmmp/qmmprc", QSettings::IniFormat);
    settings.beginGroup("MainWindow");
    //geometry
    settings.setValue("pos", this->pos());
    //last directory
    settings.setValue("last_dir",m_lastDir);
    settings.endGroup();
}

void MainWindow::showSettings()
{
    m_confDialog = new ConfigDialog(this);
    if(m_confDialog->exec() == QDialog::Accepted)
    {
        m_playlist->readSettings();
        TextScroller::getPointer()->readSettings();
    }
    delete m_confDialog;
}
