/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                      *
 *   forkotov02@hotmail.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include "output.h"
#include "decoder.h"
#include "display.h"
#include "mediafile.h"
#include "factory.h"
#include "titlebar.h"

/**
	@author Ilya Kotov <forkotov02@hotmail.ru>
*/
class PlayList;
class PlayListModel;
class ConfigDialog;
class EqWidget;
class MainVisual;


class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);

    ~MainWindow();

    PlayList *getPLPointer() { return m_playlist; }

    void seek(int);
    
public slots:
    void previous();
    void play();
    void pause();
    void stop();
    void next();

   

protected:
    virtual  void closeEvent ( QCloseEvent *);
    /*virtual  void mousePressEvent(QMouseEvent * e);
    virtual  void enterEvent ( QEvent * event );
    virtual  void leaveEvent ( QEvent * event );*/
    virtual  void changeEvent ( QEvent * event );

private slots:
    void showOutputState(OutputState *);
    void showDecoderState(DecoderState *);
    void clear();
    void startSeek();
    void endSeek();
    void showSettings();
    void addDir();
    void addFile();
    void updateEQ();

private:
    void stopAll();
    void readSettings();
    void writeSettings();
    double seeking;
    MainDisplay *display;
    PlayList *m_playlist;
    PlayListModel *m_playListModel;
    Output *output;
    QIODevice *input;
    Decoder *decoder;
    TitleBar *m_titlebar;
    ConfigDialog *m_confDialog;
    int m_preamp;
    EqWidget *m_equalizer;
    MainVisual *m_vis;
    QString m_lastDir;
};

#endif
