/***************************************************************************
 *   Copyright (C) 2006 by Ilya Kotov                                      *
 *   forkotov02@hotmail.ru                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QPainter>
#include <QResizeEvent>
#include <QSettings>
#include <QMenu>
#include <QAction>
#include <QFileDialog>

#include "dock.h"
#include "fileloader.h"
#include "playlist.h"
#include "skin.h"
#include "listwidget.h"
#include "button.h"
#include "mediafile.h"
#include "playlistmodel.h"
#include "playlisttitlebar.h"
#include "playlistslider.h"

PlayList::PlayList(QWidget *parent)
        : QWidget(parent)
{
    setWindowFlags(Qt::Dialog | Qt::FramelessWindowHint);

    m_update = FALSE;
    m_resize = FALSE;
   
    createMenus();


    resize(275,116);
    setMinimumSize(275,116);
    setBaseSize(275,116);
    m_titleBar = new PlayListTitleBar(this);
    m_titleBar->show();
    m_titleBar->move(0,0);
    //sb->move(275-25,116-25);
    m_listWidget = new ListWidget(this);
    m_listWidget->show();
    m_listWidget->setGeometry(12,20,243,58);

    m_plslider = new PlayListSlider(this);
    m_plslider->show();

    setSizeIncrement(25,29);
    m_skin = Skin::getPointer();

    m_buttonAdd = new Button(this,Skin::PL_BT_ADD,Skin::PL_BT_ADD);
    m_buttonAdd->move(11,86);
    m_buttonSub = new Button(this,Skin::PL_BT_SUB,Skin::PL_BT_SUB);
    m_buttonSub->move(41,86);
    m_selectButton  = new Button(this,Skin::PL_BT_SEL,Skin::PL_BT_SEL);
    m_selectButton->move(71,86);
    m_buttonAdd->show();
    connect (m_listWidget, SIGNAL(selectionChanged()), parent, SLOT(play()));

    connect (m_plslider, SIGNAL(sliderMoved(int)), m_listWidget, SLOT(scroll(int)));
    connect(m_listWidget, SIGNAL(positionChanged(int)), m_plslider,
            SLOT(setPos(int)));
    connect(m_skin, SIGNAL(skinChanged()), this, SLOT(update()));
    connect (m_buttonAdd, SIGNAL(clicked()), SLOT(showAddMenu()));
    connect (m_buttonSub, SIGNAL(clicked()), SLOT(showSubMenu()));
    connect (m_selectButton, SIGNAL(clicked()), SLOT(showSelectMenu()));
    setAutoFillBackground(TRUE);
    readSettings();
}


PlayList::~PlayList()
{
    //clear();
}

void PlayList::createMenus()
{
    m_addMenu = new QMenu(this);
    m_subMenu = new QMenu(this);
    m_selectMenu = new QMenu(this);
}

void PlayList::createActions()
{   //add menu
    QAction *addFileAct = new QAction("&Add File",this);
    addFileAct->setShortcut(tr("F"));
    m_addMenu->addAction(addFileAct);
    connect (addFileAct, SIGNAL(triggered()), parent(), SLOT(addFile ()));
    m_actions << addFileAct;

    QAction *addDirAct = new QAction("&Add Directory",this);
    addDirAct->setShortcut(tr("D"));
    m_addMenu->addAction(addDirAct);
    connect (addDirAct, SIGNAL(triggered()), parent(), SLOT(addDir ()));
    m_actions << addDirAct;
    //remove menu
    QAction *remSelAct = new QAction("&Remove Selected",this);
    remSelAct->setShortcut(tr("Delete"));
    m_subMenu->addAction(remSelAct);
    connect (remSelAct, SIGNAL(triggered()),
             m_playListModel, SLOT(removeSelected ()));
    this->addAction(remSelAct);

    QAction *remAllAct = new QAction("&Remove All",this);
    //remAllAct->setShortcut(tr("D")); FIXME: add correct shortcat
    m_subMenu->addAction(remAllAct);
    connect (remAllAct, SIGNAL(triggered()), m_playListModel, SLOT(clear ()));
    m_actions << remAllAct;

    QAction *remUnselAct = new QAction("&Remove Unselected",this);
    m_subMenu->addAction(remUnselAct);
    connect (remUnselAct, SIGNAL(triggered()),
             m_playListModel, SLOT(removeUnselected ()));

    //listwidget menu
    QAction *detailsAct = new QAction("&View Track Details",this);
    detailsAct->setShortcut(tr("Alt+I"));
    m_listWidget->menu()->addAction(detailsAct);
    connect (detailsAct, SIGNAL(triggered()), m_playListModel, SLOT(showDetails ()));

    m_listWidget->menu()->addSeparator();
    m_listWidget->menu()->addActions(m_subMenu->actions());
    m_actions << detailsAct;

    //select menu
    QAction *invSelAct = new QAction("Invert Selection",this);
    m_selectMenu->addAction(invSelAct);
    connect (invSelAct, SIGNAL(triggered()),
             m_playListModel, SLOT(invertSelection ()));

    m_selectMenu->addSeparator();

    QAction *selNoneAct = new QAction("&Select None",this);
    //selNoneAct->setShortcut(tr("Ctrl+Shift+A"));
    m_selectMenu->addAction(selNoneAct);
    connect (selNoneAct, SIGNAL(triggered()),
             m_playListModel, SLOT(clearSelection ()));
    this->addAction(selNoneAct);

    QAction *selAllAct = new QAction("&Select All",this);
    selAllAct->setShortcut(tr("Ctrl+A"));
    m_selectMenu->addAction(selAllAct);
    connect (selAllAct, SIGNAL(triggered()),
             m_playListModel, SLOT(selectAll ()));
    this->addAction(selAllAct);

    /*for (int i = 0; i < m_actions.size (); ++i)
    {*/
        //this->addActions(m_actions);
        //parentWidget ()->addActions(m_actions);
        Dock::getPointer()->addActions(m_actions);
    /*}*/
}

void PlayList::closeEvent(QCloseEvent*)
{
    writeSettings();
}

void PlayList::paintEvent(QPaintEvent *)
{
    int m_sx = (width()-275)/25;
    int m_sy = (height()-116)/29;
    drawPixmap(m_sx, m_sy);
}

void PlayList::drawPixmap(int sx, int sy)
{
    QPainter paint;
    paint.begin(this);
    paint.drawPixmap(0,20,m_skin->getPlPart(Skin::PL_LFILL));
    for (int i = 1; i<sy+2; i++)
    {
        paint.drawPixmap(0,20+29*i,m_skin->getPlPart(Skin::PL_LFILL));
    }
    paint.drawPixmap(0,78+29*sy,m_skin->getPlPart(Skin::PL_LSBAR));
    for (int i = 0; i<sx; i++)
    {
        paint.drawPixmap(125+i*25,78+sy*29,m_skin->getPlPart(Skin::PL_SFILL1));
    }

    paint.drawPixmap(125+sx*25,78+sy*29,m_skin->getPlPart(Skin::PL_RSBAR));
    paint.end();

}

void PlayList::resizeEvent(QResizeEvent *e)
{
    //setUpdatesEnabled(false);
    int sx = (e->size().width()-275)/25;
    int sy = (e->size().height()-116)/29;
    //qDebug("%d %d",m_sx,m_sy);
    //drawPixmap(m_sx, m_sy);

    //m_listWidget->resize(243+25*m_sx,58+25*m_sy);
    //updateList();



    //m_plslider->setUpdatesEnabled(false);
    //m_listWidget->setUpdatesEnabled(false);

    m_titleBar->resize(275+25*sx,20);
    m_plslider->resize(20,58+sy*29);

    m_listWidget->resize(243+25*sx,58+29*sy);

    m_buttonAdd->move(11,86+29*sy);
    m_buttonSub->move(41,86+29*sy);
    m_selectButton->move(71,86+29*sy);

    m_plslider->move(255+sx*25,20);
    //m_plslider->setUpdatesEnabled(true);
    //m_listWidget->setUpdatesEnabled(true);
    //setUpdatesEnabled(true);
    //setDocked(FALSE);
}
void PlayList::mousePressEvent(QMouseEvent *e)
{
    m_pos = e->pos ();
    if( (m_pos.x() > width()-25) && (m_pos.y() > height()-25))
    {
        m_resize = TRUE;
        setCursor(Qt::SizeFDiagCursor);
    }
    else
        m_resize = FALSE;
}
void PlayList::mouseMoveEvent(QMouseEvent *e)
{
    if (m_resize)
    {
        resize(e->x()+25, e->y()+25);
        //usleep(32000);
    }
}
void PlayList::mouseReleaseEvent(QMouseEvent *)
{
    setCursor(Qt::ArrowCursor);
    /*if (m_resize)
        m_listWidget->updateList();*/
    m_resize = FALSE;
    Dock::getPointer()->updateDock();
}
void PlayList::setModel(PlayListModel *model)
{
    m_playListModel = model;
    m_listWidget->setModel(model);
    createActions();
}

/*void PlayList::enterEvent(QEvent *)
{
    m_titleBar->setActive(TRUE);
}

void PlayList::leaveEvent(QEvent *)
{
    m_titleBar->setActive(FALSE);
}*/

void PlayList::changeEvent ( QEvent * event )
{
  if (event->type() == QEvent::ActivationChange)
  {
     m_titleBar->setActive(isActiveWindow());
  } 
}

void PlayList::readSettings()
{
    if (m_update)
    {
        m_listWidget->readSettings();
    }
    else
    {
        QSettings settings(QDir::homePath()+"/.qmmp/qmmprc", QSettings::IniFormat);
        settings.beginGroup("PlayList");
        //geometry
        resize(settings.value("size", QSize(275, 116)).toSize());
        move(settings.value("pos", QPoint(100, 332)).toPoint());
        settings.endGroup();
        m_update = TRUE;
    }

}

void PlayList::writeSettings()
{
    QSettings settings(QDir::homePath()+"/.qmmp/qmmprc", QSettings::IniFormat);
    settings.beginGroup("PlayList");
    //geometry
    settings.setValue("size", size());
    settings.setValue("pos", this->pos());
    settings.endGroup();
}

void PlayList::showAddMenu()
{
    m_addMenu->exec(m_buttonAdd->mapToGlobal(QPoint(0,0)));
}

void PlayList::showSubMenu()
{
    m_subMenu->exec(m_buttonSub->mapToGlobal(QPoint(0,0)));
}

void PlayList::showSelectMenu()
{
    m_selectMenu->exec(m_selectButton->mapToGlobal(QPoint(0,0)));
}
